<?php

/**
 * Main function that processes a given XLS statistics file.
 *
 * @param string $filename
 *   Full path to the XLS file to parse.
 * @return boolean
 *   TRUE if successful, FALSE on any errors or problems.
 */
function _itunesu_stats_parse_file($filename) {
  $lines = file($filename);

  // Trim lines
  foreach ($lines as &$line) {
    $line = trim($line);
  }

  // Validate first line against known values.
  $header_column_model = array (
    0 => 'ARTIST_ID',
    1 => 'ITUNES_ID',
    2 => 'ACTION_TYPE',
    3 => 'TITLE',
    4 => 'URL',
    5 => 'EPISODE_ID',
    6 => 'EPISODE_TITLE',
    7 => 'EPISODE_TYPE',
    8 => 'COUNTRY',
    9 => 'USERAGENT',
    10 => 'IP_ADDRESS',
    11 => 'TIMESTAMP',
  );
  // Shift first line (column headers) off of the $lines array.
  $header_line = array_shift($lines);
  $header_columns_received = explode("\t", $header_line);
  // Make sure AT LEAST first N columns in model match the header line in the
  // received file.
  $header_columns_received = array_slice($header_columns_received, 0, sizeof($header_column_model));
  if ($header_columns_received !== $header_column_model) {
    drupal_set_message(t('Error: column names in first line contain unexpected'
        . ' values or number of columns. Are you sure you selected the right'
        . ' file?'), 'error');
    #drupal_set_message(var_export($header_column_model, TRUE), 'error');
    #drupal_set_message(var_export($header_columns_received, TRUE), 'error');
    return FALSE;
  }

  // Process each line.
  drupal_set_message(t("File contains @num rows.", array('@num' => sizeof($lines))));
  $warnings = 0;
  foreach ($lines as $rownum => $line) {
    $cols = explode("\t", $line);
    // Trim/filter all columns
    foreach ($cols as &$col) {
      $col = trim($col);
    }
    
    // Validate each line.
    $error_msg = _itunesu_stats_validate_tsv_cols($cols);
    if ($error_msg != '') {
      drupal_set_message(t('Ignoring row @rownum because of invalid data: @msg',
          array('@rownum' => $rownum + 2, '@msg' => $error_msg)), 'warning');
      $warnings++;
      continue;
    }

    $action = $cols[2];
    $collection_actions = array('Browse', 'Subscription');
    if (in_array($action, $collection_actions)) {
      // Process Collection actions

      // Special case "Browse" action for ITUNES_ID == 0 (== channel browse)
      if ($cols[1] == "0") {
        $col_id = -1;
        $track_id = 0;
      }
      else {
        // Get collection_id
        // From stored association between Apple's "ITUNES_ID" and collection node.
        $itunes_id = $cols[1];
        $col_id = _itunesu_stats_nid_from_itunesid($itunes_id, 'ITUNES_ID');
        if (! $col_id) {
          // Try a title match.
          $collection_name = $cols[3];
          $col_id = _itunesu_stats_nid_from_title($collection_name, 'podcast_collection');

          // If title match successful, store EPISODE_ID - nid association!
          if ($col_id) {
            _itunesu_stats_store_nid_itunesid_relation($col_id, $itunes_id, 'ITUNES_ID');
          }
        }
        if (! $col_id) {
          drupal_set_message(t("Ignoring row @rownum: can't find the local collection '@name'",
            array('@rownum' => $rownum + 2, '@name' => $collection_name)), 'warning');
          $warnings++;
          continue;
        }
        $track_id = 0;
      }
    }
    else {
      // Process Item actions.

      // Map actions to our simple ones.
      if ($action == "Download") {
        $action = "Tracks";
      }
      if ($action == "DownloadAll") { 
        // TODO: Does "DownloadAll" mean "Download"??
        $action = "Tracks";
      }
      if ($action == "SubscriptionEnclosure") {
        // TODO: Does "SubscriptionEnclosure" mean "Download"??
        $action = "Tracks";
      }
      if ($action == "Stream") { // TODO: Does "Stream" mean "Preview"??
        $action = "Previews";
      }

      // Get track_id
      // From stored association between Apple's "EPISODE_ID" and collection node.
      $episode_id = $cols[5];
      $track_id = _itunesu_stats_nid_from_itunesid($episode_id, 'EPISODE_ID');
      if (! $track_id) {
        // Try a title match.
        $track_name = $cols[6];
        $track_id = _itunesu_stats_nid_from_title($track_name, 'podcast_item');

        // If title match successful, store EPISODE_ID - nid association!
        if ($track_id) {
          _itunesu_stats_store_nid_itunesid_relation($track_id, $episode_id, 'EPISODE_ID');
        }
      }
      if (! $track_id) {
        drupal_set_message(t("Ignoring row @rownum: can't find the local track '@name'",
          array('@rownum' => $rownum + 2, '@name' => $track_name)), 'warning');
        continue;
      }

      // Get collection id
      $col_id = _itunesu_stats_track_parent_nid($track_id);
    }

    // Calculate rest of data items to store.
    $timestamp = strtotime($cols[11]);
    $date = _itunesu_stats_calc_nearest_weekstart_date($timestamp);
    
    // Write to log
    _itunesu_stats_store_queue($action, $date, 1, $col_id, $track_id, $timestamp);
  }
  
  // Successfully parsed all the file. Store the parsed data.
  _itunesu_stats_store_commit();

  // Report on # of warnings found.
  if ($warnings > 0) {
    drupal_set_message(t('Note: file was processed with @num lines ignored.', 
        array('@num' => $warnings)));
  }

  return TRUE;
}

function _itunesu_stats_validate_tsv_cols($cols) {
  // Check allowed action words.
  $allowed_actions = array('Browse', 'DownloadAll', 'Download', 'Stream',
    'Subscribe', 'SubscriptionEnclosure', 'AutoDownload');
  if (! in_array($cols[2], $allowed_actions)) {
    return "Bad action '" . $cols[2] . "'";
  }

  // Check numeric fields: ITUNES_ID, ARTIST_ID and EPISODE_ID
  $numeric_check = TRUE;
  foreach (array(0,1,5) as $index) {
    if (! is_numeric($cols[$index])
        || ($cols[$index] > 0 && $cols[$index] < 1000000)) {
      $numeric_check = FALSE;
      break;
    }
  }
  if (! $numeric_check) {
    return "Bad numeric values.";
  }
  
  // Check for valid dates.
  if (strlen($cols[11]) != 28) {
    return "Bad date '" . $cols[11] . "'";
  }
  $timestamp = strtotime($cols[11]);
  if (! $timestamp || $timestamp > time()) {
    return "Bad date '" . $cols[11] . "'";
  }
  return '';
}