<?php

/**
 * Main function that processes a given XLS statistics file.
 * 
 * @param string $filename
 *   Full path to the XLS file to parse.
 * @return boolean
 *   TRUE if successful, FALSE on any errors or problems.
 */
function _itunesu_stats_parse_file($filename) {
  require_once dirname(__FILE__) . '/excel_simple_parser.php';

  $xlsparser = new ExcelSimpleParser($filename);
  if (!$xlsparser) {
    drupal_set_message(t("Internal error: could not parse file @file", array("@file" => $filename)), 'error');
    return FALSE;
  }
  $sheets = $xlsparser->getSheets();

  // Do basic check first: make sure dates match workbook names
  // Get data headers above stats on first sheet.
  $dates = $xlsparser->getCellsHoriz(4, 3, 5);

  // Compare against sheet names: there should be 4 sheets per date
  $ok = TRUE;
  $counts_per_date = array();
  $tot_sheets = 0;
  #print_r($sheets);
  foreach ($sheets as $sheet_index => $sheet_name) {
    if ($sheet_index == 0) {
      if ($sheet_name != "Summary") {
        $ok = FALSE;
        break;
      }
      continue;
    }
    elseif (!in_array(substr($sheet_name, 0, 10), $dates)) {
      $ok = FALSE;
      break;
    }
    $counts_per_date[substr($sheet_name, 0, 10)]++;
    $tot_sheets++;
  }
  // Make sure counts are ok.
  if ($ok) {
    if ($tot_sheets != sizeof($counts_per_date) * 5) {
      drupal_set_message(t("Missing sheets for one or more dates"), 'error');
      return FALSE;
    }
  }
  if (!$ok) {
    drupal_set_message(t("Worksheets' names don't match dates."), 'error');
    return FALSE;
  }

  // Process first sheet: "Summary"
  $cells = $xlsparser->getCellsRegion(5, 15, 2, 7);
  // Collection id of -1 means "all collections combined"
  $col_id = -1;
  #print_r($cells);
  foreach ($cells as $row) {
    // First cell in row is User Action name
    $action = array_shift($row);
    // Rest is one value per date
    foreach ($row as $index => $value) {
      // Generate timestamp from date.
      $timestamp = strtotime($dates[$index]);
      _itunesu_stats_store_queue($action, $dates[$index], $value, -1, 0, $timestamp);
    }
  }

  // Process sheets for each date for detailed stats.
  foreach ($dates as $date) {
    // Downloaded tracks, by track & collection
    $ok1 = _itunesu_stats_process_tracks("Tracks", $xlsparser, $date);
    // Previewed tracks, by track & collection
    $ok2 = _itunesu_stats_process_tracks("Previews", $xlsparser, $date);
    // Browsing in collections
    $ok3 = _itunesu_stats_process_browse($xlsparser, $date);
    if (!$ok1 || !$ok2 || !$ok3) {
      return FALSE;
    }
  }
  // Successfully parsed all the file. Store it!
  _itunesu_stats_store_commit();

  return TRUE;
}


/**
 * Helper function to _itunesu_stats_parse_file(). Process "x-x-x Tracks" and
 * "x-x-x Previews" sheets in an XLS file.
 *
 * @param string $action
 * @param ExcelSimpleParser $e
 * @param string $date
 * @return boolean
 */
function _itunesu_stats_process_tracks($action, $e, $date) {
  // Process "$action" sheet for this date.
  $sheet_name = "$date $action";
  #echo "---$sheet_name--------\n";
  $ok = $e->setSheetByName($sheet_name);
  if (!$ok) {
    drupal_set_message(t("Couldn't find sheet @name",
        array('@action' => $action, '@name' => $sheet_name)));
    return FALSE;
  }
  $cells = $e->getCellsAll();
  #print_r($cells);
  // Basic-check the headers
  $headers = array_shift($cells);
  if (!_itunesu_stats_basic_check($headers, $action)) {
    drupal_set_message(t("Sheet @name: first row didn't match expected values.",
        array('@action' => $action, '@name' => $sheet_name)));
    return FALSE;
  }
  $deleted = 0;
  // Ok, write stats
  foreach ($cells as $rownum => $row) {
    // Get the track ID from the GUID
    // e.g "258 at http://prod71ws.ruv.itesm.mx/apps/itunes"
    list($track_id,) = explode(' ', $row[3]);
    // Skip empty track definitions (no way to track them)
    if ($track_id == '') {
      $deleted++;
      continue;
    }
    // If not numeric, try to get nid from URL
    if (!is_numeric($track_id)) {
      $track_id = _itunesu_stats_nid_from_url($row[3], 'field_link_url', 'podcast_item');
      if (!$track_id) {
        // Try to get from first column in this row: "path"
        #dsm($row[0]);
        $tmp = explode('>', $row[0]);
        $track_name = trim($tmp[2]);
        $track_id = _itunesu_stats_nid_from_title($track_name, 'podcast_item');
        #dsm("Track name: " . filter_xss($track_name) . ", track_id = $track_id");
        if ($track_id) {
          // Make sure the collection matches too.
          $col_name = trim($tmp[1]);
          $col_id = _itunesu_stats_nid_from_title($col_name, 'podcast_collection');
          #dsm("Collection name: " . filter_xss($col_name) . ", track_id = $col_id");
          if ($col_id) {
            $tmp_parent = _itunesu_stats_track_parent_nid($track_id);
            if ($col_id != $tmp_parent) {
              $track_id = FALSE;
            }
          }
        }
        if (!$track_id) {
          drupal_set_message(t('Sheet "@name", row @rownum: can\'t find the local track definition for reported GUID @url',
            array('@rownum' => $rownum + 2, '@name' => $sheet_name, '@url' => $row[3])), 'warning');
          #drupal_set_message(t('Skipping track @url', array('@url' => $row[3])));
          continue;
          # TODO: Fail or continue?
        }
      }
    }
    // Get collection ID.
    $col_id = _itunesu_stats_track_parent_nid($track_id);
    if ($col_id && $track_id) {
      // Generate timestamp from date.
      $timestamp = strtotime($date);
      // Write to log
      _itunesu_stats_store_queue($action, $date, $row[1], $col_id, $track_id, $timestamp);
    }
  }
  if ($deleted) {
    drupal_set_message(t('Sheet "@name": ignored @num rows with deleted tracks.', array('@name' => $sheet_name, '@num' => $deleted)));
  }
  return TRUE;
}

/**
 * Helper function to _itunesu_stats_parse_file(). Process "x-x-x Browse" sheet.
 *
 * @param ExcelSimpleParser $e
 * @param string $date
 * @return boolean
 */
function _itunesu_stats_process_browse($e, $date) {
  static $error_shown_flags;

  // Process "$action" sheet for this date.
  $sheet_name = "$date Browse";
  #echo "---$sheet_name--------\n";
  $ok = $e->setSheetByName($sheet_name);
  if (!$ok) {
    drupal_set_message(t("Couldn't find sheet @name",
        array('@action' => 'Browse', '@name' => $sheet_name)));
    return FALSE;
  }
  $cells = $e->getCellsAll();
  #print_r($cells);
  // Basic-check the headers
  $headers = array_shift($cells);
  if (!_itunesu_stats_basic_check($headers, 'Browse')) {
    drupal_set_message(t("Sheet @name: first row didn't match expected values.",
        array('@action' => 'Browse', '@name' => $sheet_name)));
    return FALSE;
  }
  // Ok, write stats
  foreach ($cells as $row) {
    $col_id = 0;

    // Handle cases for specific collections.
    // E.g. "Tecnológico de Monterrey > Cátedra Alfonso Reyes"
    $tmp = explode('>', $row[0]);
    if (sizeof($tmp) == 2) {
      $col_name = trim($tmp[1]);
      // Convert into numeric col_id
      $col_id = _itunesu_stats_nid_from_title($col_name, 'podcast_collection');
      // If can't parse into a collection nid, report error and break.
      if (!$col_id) {
        if (!isset($error_shown_flags[$col_name])) {
          drupal_set_message(t("Can't find a collection named %name. Are you sure you selected the right file?", array('%name' => $col_name)), 'warning');
          // Set flag so we don't show this error again during processing.
          $error_shown_flags[$col_name] = TRUE;
        }
        continue;
        # OK To continue? Or quit?
      }
    }
    else {
      // Skip others.
      // TODO: XLS seems to have lines for "Institution name", but does not match the browse totals in the "Summary" sheet for that date.
      continue;
    }
    // Generate timestamp from date.
    $timestamp = strtotime($date);
    // Write to log
    _itunesu_stats_store_queue("Browse", $date, $row[1], $col_id, 0, $timestamp);
  }
  return TRUE;
}

/**
 * Helper function that runs basic checks on incoming XLS headers against known
 * strings.
 *
 * @param array $headers
 *   Array of strings, one element per header from the incoming XLS file.
 * @param string $action
 *   Either 'Tracks', 'Previews' or 'Browse'.
 * @return boolean
 */
function _itunesu_stats_basic_check($headers, $action) {
  switch ($action) {
    case 'Tracks':
    case 'Previews':
      $model = array('Path', 'Count', 'Handle', 'GUID');
      break;
    case 'Browse':
      $model = array('Path', 'Count', 'Handle', 'GUID');
      break;
    default:
      return FALSE;
  }
  if ($headers != $model) {
    return FALSE;
  }
  return TRUE;
}
